Unit Mouse;

Interface

Uses Crt, Dos, Graph;

Type

 Ware = (Hard, Soft);
 SpaceS = Array [0 .. 1, 0 .. 15] of Word;

{ Definicao dos shapes para redefinicao do crusor (SetGraphicsCursor) }

Const

  { Hot Point = 6,5 }

  Nave:SpaceS = (($FFFF, $FFFF, $FFFF, $FFFF,
                  $FFFF, $F00F, $0000, $0000,
                  $0000, $0000, $F00F, $FFFF,
                  $FFFF, $FFFF, $FFFF, $FFFF),
                 ($0000, $0000, $0000, $0000,
                  $0000, $0000, $07E0, $7FFE,
                  $7FFE, $07E0, $0000, $0000,
                  $0000, $0000, $0000, $0000));

  { Hot Point = 7,5 }

  Cruz:SpaceS = (($FFFF, $FFFF, $FEFF, $FEFF,
                  $FEFF, $FEFF, $FEFF, $FEFF,
                  $8103, $FFFF, $FEFF, $FEFF,
                  $FEFF, $FEFF, $FEFF, $FFFF),
                 ($0000, $0000, $0100, $0100,
                  $0100, $0100, $0100, $0100,
                  $7EFC, $0100, $0100, $0100,
                  $0100, $0100, $0100, $0000));

  { Hot Point = 1,1 }

  Mao:SpaceS = (($DFFF, $1FFF, $0F3F, $863F,
                 $C03F, $E03F, $E03F, $C03F,
                 $803F, $803F, $C03F, $E03F,
                 $F0FF, $FFFF, $FFFF, $FFFF),
                ($4000, $A000, $90C0, $4900,
                 $2640, $1240, $1040, $2040,
                 $4040, $4040, $2040, $1080,
                 $0F00, $0000, $0000, $0000));

  { Hot Point = 1,0 }

  Seta:SpaceS = (($1FFF, $1FFF, $0FFF, $0FFF,
                  $07FF, $07FF, $03FF, $03FF,
                  $01FF, $01FF, $01FF, $C3FF,
                  $C3FF, $C3FF, $E1FF, $E1FF),
                 ($8000, $C000, $C000, $E000,
                  $E000, $F000, $F000, $F800,
                  $F800, $FC00, $1000, $1000,
                  $1800, $0800, $0800, $0C00));


{ Rotinas a serem exportadas pela unit }


Procedure ResetMouse;
Procedure EnableMouse;
Procedure DisableMouse;
Procedure SetHerculesMouse;
Procedure ReadMouse (Var Cdx, Cdy: Integer; Var Button: Word);
Procedure SetCursorLocation (Cdx, Cdy: Word);
Procedure SetGraphicsCursor (Cdx, Cdy: Integer; Var Shape: SpaceS);
Procedure WaitMouse (Var Cx0, Cy0: Integer; Var Button: Word);
Procedure WaitPress (Var Button: Word);
Procedure WaitRelease;

Var

  MouseInit: Byte;


Implementation

{ Implementacao das rotinas de manipulacao do mouse }

Var Regs: Registers;

{ Inicializa o mouse }

Procedure ResetMouse;

Var

  I: Integer;

Begin
  With Regs do Begin
    AX := 0;
    BX := 0;
    Intr (51, Regs);
    MouseInit := Bl;
    if AX = 0 Then
      MouseInit := MouseInit AND $7F
    Else
      MouseInit := MouseInit OR $80;
  End;
End;

{ Mostra o cursor na tela }

Procedure EnableMouse;

Begin
  Regs.AX := 1;
  Intr (51, Regs);
End;

{ Retina o cursor da tela }

Procedure DisableMouse;

Begin
  Regs.AX := 2;
  Intr (51, Regs);
End;

{ Le posicao do mouse na tela e estado dos botoes naquele momento }

Procedure ReadMouse (Var Cdx, Cdy: Integer; Var Button: Word);

Begin
  With Regs do Begin
    AX := 3;
    Intr (51, Regs);
    Cdx := CX;
    Cdy := DX;
    Button := BX;
  End;
End;

{ Posiciona o cursor em determinada posicao da tela.
  Independe de movimentos do mouse }

Procedure SetCursorLocation (Cdx, Cdy: Word);

Begin
  With Regs do Begin
    AX := 4;
    CX := Cdx;
    DX := Cdy;
    Intr (51, Regs);
  End;
End;

{ Define novo shape do mouse, de acordo com padrao definido no
  inicio do programa. }

Procedure SetGraphicsCursor (Cdx, Cdy: Integer; Var Shape: SpaceS);

Begin
  With Regs do Begin
    AX := 9;
    BX := Cdx;
    CX := Cdy;
    DX := Ofs (Shape);
    ES := Seg (Shape);
    Intr (51, Regs);
  End;
End;

{ Espera ate que o mouse seja movimentado ou algum botao pressionado
  e devolve sua posicao e estado dos botoes. }

Procedure WaitMouse (Var Cx0, Cy0: Integer; Var Button: Word);

Var

  Cx1, Cy1: Integer;

Begin
  ReadMouse (Cx0, Cy0, Button);
  While Button < 0 do ReadMouse (Cx0, Cy0, Button);
  Cx1 := Cx0;
  Cy1 := Cy0;
  While NOT KeyPressed AND (Cx0 = Cx1) AND (Cy0 = Cy1) AND (Button = 0) do
    ReadMouse (Cx0, Cy0, Button);
End;

{ Aguarda um botao e devolve estado }

Procedure WaitPress (Var Button: Word);

Var

  Cx, Cy: Integer;
  Ch: Char;

Begin
  ReadMOuse (Cx, Cy, Button);
  While Button < 0 do REadMouse (Cx, Cy, Button);
  While NOT Keypressed And (Button = 0) do
    ReadMouse (Cx, Cy, Button);
  While KeyPressed Do Ch := ReadKey;
End;

{ Aguarda que todos os botoes estejam livres }

Procedure WaitRelease;

Var

  Bt: Word;
  Cx, Cy: Integer;

Begin
  ReadMouse (Cx, Cy, Bt);
  While Bt < 0 do ReadMouse (Cx, Cy, Bt);
End;

{ Adapta o cursor ao padrao grafico da placa Hercules }

Procedure SetHerculesMouse;

Begin
  Mem [$40:$49] := 6;
End;

Begin
  ResetMouse;
End.















