Unit VGA; { Implementacao de funcoes para acessar os modos da placa TRIDENT }

Interface

Uses Dos, Crt;


Const

  MCGA: Integer = $13;
  SVGAMED: Integer = $5E;
  SVGAHI: Integer = $62;

Procedure Inic_Modo (Modo: Integer);

Procedure SetMCGA (x, y, Cor: Integer);
Procedure SetSVGAMED (x, y, Cor: Integer);
Procedure SetSVGAHI (x, y, Cor: Integer);

Function  GetSVGAHI (x, y: Integer): Byte;

Procedure Muda_64_Tons_Cinza_Hi;

Procedure Fim_Modo_Graf;

Implementation

Var

  Bank_Ant: Byte;

(* Inic_Modo: Inicializa o modo passado por parametro.
   Recebe: Um inteiro indicando o modo a ser setado.
   Retorna: Nada. *)

Procedure Inic_Modo (modo: Integer);

Var

  Regs: Registers;

Begin

  Bank_Ant := 0;

  Regs.AH := 0;
  Regs.AL := modo;
  Intr ($10, Regs);

End;

(* SetMCGA: Coloca um pixel no modo grafico MCGA (320 x 200 com 256 cores).
   Recebe: As coordenadas os inteiros correspondentes as coordenadas do
   video e a cor do pixel.
   Retonra: Nada. *)

Procedure SetMCGA (x, y, Cor: Integer);

Var

  Tela: Array [0 .. 65534] of Byte Absolute $A000:$0000;

  OffSet: Word;

Begin

  OffSet := y * 320 + x;

  Tela [OffSet] := Cor;

End;


(* SetSVGAMED: Coloca um pixel no modo grafico SVGAMED (800 x 600 com 256
   cores).
   Recebe: As coordenadas os inteiros correspondentes as coordenadas do
   video e a cor do pixel.
   Retonra: Nada. *)

Procedure SetSVGAMED (x, y, cor: Integer);

Var

  Tela: Array [0 .. 65534] of Byte Absolute $A000:$0000;

  OffSet: Longint;

  Bank: Byte;

Begin

  OffSet := 800;
  OffSet := y * OffSet + x;

  Bank := OffSet SHR 16;
  OffSet := OffSet AND $FFFF;

  If (Bank <> Bank_Ant) Then
    Begin

      Asm

        cli
        Push   AX
        Push   DX
        Mov    AH, Bank
        Xor    AH, 2
        Mov    DX, 3C4h
        Mov    AL, 0Eh
        Out    DX, AX
        Pop DX
        Pop AX
        sti

      End;

      Bank_Ant := Bank;

    End;

  Tela [OffSet] := Cor;

End;

(* SetSVGAHI: Coloca um pixel no modo grafico SVGAMED (1024 x 768 com 256
   cores).
   Recebe: As coordenadas os inteiros correspondentes as coordenadas do
   video e a cor do pixel.
   Retorna: Nada. *)

Procedure SetSVGAHI (x, y, cor: Integer);

Var

  Tela: Array [0 .. 65534] of Byte Absolute $A000:$0000;

  OffSet: Longint;

  Bank: Byte;

Begin

  OffSet := 1024;
  OffSet := y * OffSet + x;

  Bank := OffSet SHR 16;
  OffSet := OffSet AND $FFFF;

  If (Bank <> Bank_Ant) Then
    Begin

      Asm

        cli
        Push   AX
        Push   DX
        Mov    AH, Bank
        Xor    AH, 2
        Mov    DX, 3C4h
        Mov    AL, 0Eh
        Out    DX, AX
        Pop DX
        Pop AX
        sti

      End;

      Bank_Ant := Bank;

    End;

  Tela [OffSet] := Cor;

End;

(* GetSVGAHI: Informa a cor de um pixel no modo grafico SVGAMED
   (1024 x 768 com 256 cores).
   Recebe: As coordenadas os inteiros correspondentes as coordenadas do
   video.
   Retorna: A cor do pixel. *)

Function  GetSVGAHI (x, y: Integer): Byte;

Var

  Tela: Array [0 .. 65534] of Byte Absolute $A000:$0000;

  OffSet: Longint;

  Bank: Byte;

Begin

  OffSet := 1024;
  OffSet := y * OffSet + x;

  Bank := OffSet SHR 16;
  OffSet := OffSet AND $FFFF;

  If (Bank <> Bank_Ant) Then
    Begin

      Asm

        cli
        Push   AX
        Push   DX
        Mov    AH, Bank
        Xor    AH, 2
        Mov    DX, 3C4h
        Mov    AL, 0Eh
        Out    DX, AX
        Pop DX
        Pop AX
        sti

      End;

      Bank_Ant := Bank;

    End;

  GetSVGAHI := Tela [OffSet];

End;

(* Muda_64_Tons_Cinza_Hi: muda as 64 primeiras cores da palete dos modos HI
   para os 64 tons possiveis de cinza.
   Recebe: Nada;
   Retorna: Nada; *)
{
Procedure Muda_64_Tons_Cinza_HI;

Var

  Regs: Registers;
  i: Integer;
  Palette: Array [1 .. 256,1 .. 3] of Integer;

Begin

  for i := 1 to 65 do Begin
    Palette [i, 1] := i;
    Palette [i, 2] := i;
    Palette [i, 3] := i;
  End;
  Asm
    push dx
    push bx
    push cx
    push ax
    push es

    mov ax, ds
    mov es, ax
    lea  dx, [Palette]

    mov  bx, 256
    mov  cx, 0;
    mov ax, 1012h
    int 10h
    pop es
    pop  ax
    pop cx
    pop bx
    pop dx

  End;

End;
}

(* Muda_64_Tons_Cinza_Hi: muda as 64 primeiras cores da palete dos modos HI
   para os 64 tons possiveis de cinza.
   Recebe: Nada;
   Retorna: Nada; *)

Procedure Muda_64_Tons_Cinza_HI;

Var

  Regs: Registers;
  i: Integer;

Begin

  for i := 0 to 64 do Begin
    Regs.AH := $10;
    Regs.AL := $10;
    Regs.BX := i;
    Regs.CH := Regs.BL;
    Regs.CL := Regs.BL;
    Regs.DH := Regs.BL;
    Regs.DL := Regs.BL;
    Intr ($10, Regs);
  End
End;


(* Fim_Modo_Graf: Retorna ao modo vga texto (80 x 25 colunas texto).
   Recebe: Nada.
   Retorna: Nada. *)

Procedure Fim_Modo_Graf;

Var

  Regs: Registers;

Begin

  Regs.AH := 0;
  Regs.AL := 3; (* Modo de 80 linhas, 25 colunas modo texto. *)
  Intr ($10, Regs);

End;

End.
